@@ -1,14 +1,20 @@
 from zep_python.exceptions import APIError, NotFoundError
-from zep_python.models import Memory, Message, SearchPayload, SearchResult, Summary
+from zep_python.models import (
+    Memory,
+    MemorySearchPayload,
+    MemorySearchResult,
+    Message,
+    Summary,
+)
 from zep_python.zep_client import ZepClient
 
 __all__ = [
     "ZepClient",
     "Memory",
     "Message",
-    "SearchPayload",
+    "MemorySearchPayload",
     "APIError",
     "NotFoundError",
-    "SearchResult",
-    "Summary"
+    "MemorySearchResult",
+    "Summary",
 ]
