@@ -74,6 +74,7 @@ class Message(BaseModel):
     uuid: Optional[str] = Field(optional=True, default=None)
     created_at: Optional[str] = Field(optional=True, default=None)
     token_count: Optional[int] = Field(optional=True, default=None)
+    metadata: Optional[Dict[str, Any]] = Field(optional=True, default=None)
 
     def to_dict(self) -> Dict[str, Any]:
         """
@@ -91,19 +92,25 @@ class Memory(BaseModel):
     """
     Represents a memory object with messages, metadata, and other attributes.
 
-    :param messages: A list of message objects, where each message contains a role and
-                     content.
-    :type messages: Optional[List[Dict[str, Any]]]
-    :param metadata: A dictionary containing metadata associated with the memory.
-    :type metadata: Optional[Dict[str, Any]]
-    :param summary: A Summary object.
-    :type summary: Optional[Summary]
-    :param uuid: A unique identifier for the memory.
-    :type uuid: Optional[str]
-    :param created_at: The timestamp when the memory was created.
-    :type created_at: Optional[str]
-    :param token_count: The token count of the memory.
-    :type token_count: Optional[int]
+    Attributes
+    ----------
+    messages : Optional[List[Dict[str, Any]]]
+        A list of message objects, where each message contains a role and content.
+    metadata : Optional[Dict[str, Any]]
+        A dictionary containing metadata associated with the memory.
+    summary : Optional[Summary]
+        A Summary object.
+    uuid : Optional[str]
+        A unique identifier for the memory.
+    created_at : Optional[str]
+        The timestamp when the memory was created.
+    token_count : Optional[int]
+        The token count of the memory.
+
+    Methods
+    -------
+    to_dict() -> Dict[str, Any]:
+        Returns a dictionary representation of the message.
     """
 
     messages: List[Message] = Field(
@@ -119,23 +126,23 @@ class Memory(BaseModel):
         return self.dict()
 
 
-class SearchPayload(BaseModel):
+class MemorySearchPayload(BaseModel):
     """
     Represents a search payload for querying memory.
 
     Attributes
     ----------
-    meta : Dict[str, Any]
+    metadata : Dict[str, Any]
         Metadata associated with the search query.
     text : str
         The text of the search query.
     """
 
     text: str = Field("A text is required")
-    meta: Optional[Dict[str, Any]] = Field(optional=True, default=None)
+    metadata: Optional[Dict[str, Any]] = Field(optional=True, default=None)
 
 
-class SearchResult(BaseModel):
+class MemorySearchResult(BaseModel):
     """
     Represents a search result from querying memory.
 
@@ -143,10 +150,8 @@ class SearchResult(BaseModel):
     ----------
     message : Optional[Dict[str, Any]]
         The message associated with the search result.
-    meta : Optional[Dict[str, Any]]
+    metadata : Optional[Dict[str, Any]]
         Metadata associated with the search result.
-    score : Optional[float]
-        The score of the search result.
     summary : Optional[str]
         The summary of the search result.
     dist : Optional[float]
@@ -154,7 +159,6 @@ class SearchResult(BaseModel):
     """
 
     message: Optional[Dict[str, Any]] = None
-    meta: Optional[Dict[str, Any]] = None
-    score: Optional[float] = None
+    metadata: Optional[Dict[str, Any]] = None
     summary: Optional[str] = None
     dist: Optional[float] = None
