@@ -1,16 +1,20 @@
 from __future__ import annotations
 
-import asyncio
 from types import TracebackType
-from typing import List, Optional, Type
+from typing import Any, Dict, List, Optional, Type
 
 import httpx
 
 from zep_python.exceptions import APIError, NotFoundError
-from zep_python.models import Memory, Message, SearchPayload, SearchResult, Summary
-from zep_python.utils import sync
+from zep_python.models import (
+    Memory,
+    MemorySearchPayload,
+    MemorySearchResult,
+    Message,
+    Summary,
+)
 
-API_BASEURL = "/api/v1"
+API_BASE_PATH = "/api/v1"
 
 
 class ZepClient:
@@ -48,40 +52,70 @@ class ZepClient:
         base_url : str
             The base URL of the API.
         """
-        self.base_url = base_url
-        self.client = httpx.AsyncClient()
+        self.base_url = f"{base_url}{API_BASE_PATH}"
+        self.aclient = httpx.AsyncClient(base_url=self.base_url)
+        self.client = httpx.Client(base_url=self.base_url)
 
-    # Asynchronous context manager entry point
     async def __aenter__(self) -> "ZepClient":
+        """Asynchronous context manager entry point"""
         return self
 
-    # Sync context manager exit point
     async def __aexit__(
         self,
         exc_type: Type[Exception],
         exc_val: Exception,
         exc_tb: TracebackType,
     ) -> None:
+        """Asynchronous context manager exit point"""
         await self.aclose()
 
-    # Asynchronous context manager entry point
     def __enter__(self) -> "ZepClient":
+        """Sync context manager entry point"""
         return self
 
-    # Sync context manager exit point
     def __exit__(
         self,
         exc_type: Type[Exception],
         exc_val: Exception,
         exc_tb: TracebackType,
     ) -> None:
+        """Sync context manager exit point"""
         self.close()
 
-    @sync
+    def _parse_get_memory_response(self, response_data: Any) -> Memory:
+        """Parse the response from the get_memory API call."""
+        messages: List[Message]
+        try:
+            messages = [
+                Message.parse_obj(m) for m in response_data.get("messages", None)
+            ]
+            if len(messages) == 0:
+                raise ValueError("Messages can't be empty")
+        except (TypeError, ValueError) as e:
+            raise APIError(message="Unexpected response format from the API") from e
+
+        summary: Optional[Summary] = None
+        if response_data.get("summary", None) is not None:
+            summary = Summary.parse_obj(response_data["summary"])
+
+        memory = Memory(
+            messages=messages,
+            # Add the 'summary' field if it is present in the response.
+            summary=summary,
+            # Add any other fields from the response that are relevant to the
+            # Memory class.
+        )
+        return memory
+
+    def _gen_get_params(self, lastn: Optional[int] = None) -> Dict[str, Any]:
+        params = {}
+        if lastn is not None:
+            params["lastn"] = lastn
+        return params
+
     def get_memory(self, session_id: str, lastn: Optional[int] = None) -> Memory:
         """
-        Retrieve memory for the specified session. This method is a synchronous wrapper
-        for the asynchronous method `aget_memory`.
+        Retrieve memory for the specified session.
 
         Parameters
         ----------
@@ -93,20 +127,31 @@ class ZepClient:
 
         Returns
         -------
-        List[Memory]
-            A list of Memory objects representing the retrieved memory entries.
+        Memory
+            A memory object containing a Summary, metadata, and list of Messages.
 
         Raises
         ------
         APIError
             If the API response format is unexpected.
         """
-        # we've wrapped the function in a decorator that will run it synchronously.
-        # ignore the type error.
+
         if session_id is None or session_id == "":
             raise ValueError("session_id must be provided")
 
-        return self.aget_memory(session_id, lastn)  # type: ignore
+        url = f"/sessions/{session_id}/memory"
+        params = self._gen_get_params(lastn)
+        response = self.client.get(url, params=params)
+
+        if response.status_code == 404:
+            raise NotFoundError(f"No memory found for session {session_id}")
+
+        if response.status_code != 200:
+            raise APIError(response)
+
+        response_data = response.json()
+
+        return self._parse_get_memory_response(response_data)
 
     async def aget_memory(self, session_id: str, lastn: Optional[int] = None) -> Memory:
         """
@@ -123,7 +168,7 @@ class ZepClient:
         Returns
         -------
         Memory
-            A Memory object representing the retrieved memory entries.
+            A memory object containing a Summary, metadata, and list of Messages.
 
         Raises
         ------
@@ -133,47 +178,23 @@ class ZepClient:
         if session_id is None or session_id == "":
             raise ValueError("session_id must be provided")
 
-        url = f"{self.base_url}{API_BASEURL}/sessions/{session_id}/memory"
-        params = (
-            {"lastn": lastn} if lastn is not None else {}
-        )  # Include 'lastn' as a query parameter if provided
-        response = await self.client.get(url, params=params)
+        url = f"/sessions/{session_id}/memory"
+        params = self._gen_get_params(lastn)
+        response = await self.aclient.get(url, params=params)
 
         if response.status_code == 404:
             raise NotFoundError(f"No memory found for session {session_id}")
 
         if response.status_code != 200:
-            raise APIError(f"Unexpected status code: {response.status_code}")
+            raise APIError(response)
 
         response_data = response.json()
-        messages: List[Message]
-        try:
-            messages = [
-                Message.parse_obj(m) for m in response_data.get("messages", None)
-            ]
-            if len(messages) == 0:
-                raise ValueError("Messages can't be empty")
-        except (TypeError, ValueError) as e:
-            raise APIError("Unexpected response format from the API") from e
 
-        summary: Optional[Summary] = None
-        if response_data.get("summary", None) is not None:
-            summary = Summary.parse_obj(response_data["summary"])
+        return self._parse_get_memory_response(response_data)
 
-        memory = Memory(
-            messages=messages,
-            # Add the 'summary' field if it is present in the response.
-            summary=summary,
-            # Add any other fields from the response that are relevant to the
-            # Memory class.
-        )
-        return memory
-
-    @sync
     def add_memory(self, session_id: str, memory_messages: Memory) -> str:
         """
-        Add memory to the specified session. This method is a synchronous wrapper for
-        the asynchronous method `aadd_memory`.
+        Add memory to the specified session.
 
         Parameters
         ----------
@@ -192,12 +213,17 @@ class ZepClient:
         APIError
             If the API response format is unexpected.
         """
-        # we've wrapped the function in a decorator that will run it synchronously.
-        # ignore the type error.
         if session_id is None or session_id == "":
             raise ValueError("session_id must be provided")
 
-        return self.aadd_memory(session_id, memory_messages)  # type: ignore
+        response = self.client.post(
+            f"/sessions/{session_id}/memory",
+            json=memory_messages.dict(exclude_none=True),
+        )
+        if response.status_code != 200:
+            raise APIError(response)
+
+        return response.text
 
     async def aadd_memory(self, session_id: str, memory_messages: Memory) -> str:
         """
@@ -223,20 +249,18 @@ class ZepClient:
         if session_id is None or session_id == "":
             raise ValueError("session_id must be provided")
 
-        response = await self.client.post(
-            f"{self.base_url}{API_BASEURL}/sessions/{session_id}/memory",
+        response = await self.aclient.post(
+            f"/sessions/{session_id}/memory",
             json=memory_messages.dict(exclude_none=True),
         )
         if response.status_code != 200:
-            raise APIError(f"Unexpected status code: {response.status_code}")
+            raise APIError(response)
 
         return response.text
 
-    @sync
     def delete_memory(self, session_id: str) -> str:
         """
-        Delete memory for the specified session. This method is a synchronous wrapper
-        for the asynchronous method `adelete_memory`.
+        Delete memory for the specified session.
 
         Parameters
         ----------
@@ -253,12 +277,16 @@ class ZepClient:
         APIError
             If the API response format is unexpected.
         """
-        # we've wrapped the function in a decorator that will run it synchronously.
-        # ignore the type error.
         if session_id is None or session_id == "":
             raise ValueError("session_id must be provided")
 
-        return self.adelete_memory(session_id)  # type: ignore
+        response = self.client.delete(f"/sessions/{session_id}/memory")
+        if response.status_code == 404:
+            raise NotFoundError(f"No session found for session {session_id}")
+
+        if response.status_code != 200:
+            raise APIError(response)
+        return response.text
 
     async def adelete_memory(self, session_id: str) -> str:
         """
@@ -282,74 +310,35 @@ class ZepClient:
         if session_id is None or session_id == "":
             raise ValueError("session_id must be provided")
 
-        response = await self.client.delete(
-            f"{self.base_url}{API_BASEURL}/sessions/{session_id}/memory"
-        )
+        response = await self.aclient.delete(f"/sessions/{session_id}/memory")
         if response.status_code == 404:
             raise NotFoundError(f"No session found for session {session_id}")
 
         if response.status_code != 200:
-            raise APIError(f"Unexpected status code: {response.status_code}")
+            raise APIError(response)
         return response.text
 
-    @sync
     def search_memory(
         self,
         session_id: str,
-        search_payload: SearchPayload,
+        search_payload: MemorySearchPayload,
         limit: Optional[int] = None,
-    ) -> List[SearchResult]:
+    ) -> List[MemorySearchResult]:
         """
-        Search memory for the specified session. This method is a synchronous wrapper
-        for the asynchronous method `asearch_memory`.
+        Search memory for the specified session.
 
         Parameters
         ----------
         session_id : str
             The ID of the session for which memory should be searched.
-        search_payload : SearchPayload
+        search_payload : MemorySearchPayload
             A SearchPayload object representing the search query.
         limit : Optional[int], optional
             The maximum number of search results to return. Defaults to None (no limit).
 
         Returns
         -------
-        List[SearchResult]
-            A list of SearchResult objects representing the search results.
-
-        Raises
-        ------
-        APIError
-            If the API response format is unexpected.
-        """
-        # we've wrapped the function in a decorator that will run it synchronously.
-        # ignore the type error.
-        if session_id is None or session_id == "":
-            raise ValueError("session_id must be provided")
-
-        return self.asearch_memory(session_id, search_payload, limit)  # type: ignore
-
-    async def asearch_memory(
-        self,
-        session_id: str,
-        search_payload: SearchPayload,
-        limit: Optional[int] = None,
-    ) -> List[SearchResult]:
-        """
-        Asynchronously search memory for the specified session.
-
-        Parameters
-        ----------
-        session_id : str
-            The ID of the session for which memory should be searched.
-        search_payload : SearchPayload
-            A SearchPayload object representing the search query.
-        limit : Optional[int], optional
-            The maximum number of search results to return. Defaults to None (no limit).
-
-        Returns
-        -------
-        List[SearchResult]
+        List[MemorySearchResult]
             A list of SearchResult objects representing the search results.
 
         Raises
@@ -364,16 +353,66 @@ class ZepClient:
             raise ValueError("search_payload must be provided")
 
         params = {"limit": limit} if limit is not None else {}
-        response = await self.client.post(
-            f"{self.base_url}{API_BASEURL}/sessions/{session_id}/search",
+        response = self.client.post(
+            f"/sessions/{session_id}/search",
             json=search_payload.dict(),
             params=params,
         )
         if response.status_code == 404:
             raise NotFoundError("No query results found")
         if response.status_code != 200:
-            raise APIError(f"Unexpected status code: {response.status_code}")
-        return [SearchResult(**search_result) for search_result in response.json()]
+            raise APIError(response)
+        return [
+            MemorySearchResult(**search_result) for search_result in response.json()
+        ]
+
+    async def asearch_memory(
+        self,
+        session_id: str,
+        search_payload: MemorySearchPayload,
+        limit: Optional[int] = None,
+    ) -> List[MemorySearchResult]:
+        """
+        Asynchronously search memory for the specified session.
+
+        Parameters
+        ----------
+        session_id : str
+            The ID of the session for which memory should be searched.
+        search_payload : MemorySearchPayload
+            A SearchPayload object representing the search query.
+        limit : Optional[int], optional
+            The maximum number of search results to return. Defaults to None (no limit).
+
+        Returns
+        -------
+        List[MemorySearchResult]
+            A list of SearchResult objects representing the search results.
+
+        Raises
+        ------
+        APIError
+            If the API response format is unexpected.
+        """
+        if session_id is None or session_id == "":
+            raise ValueError("session_id must be provided")
+
+        if search_payload is None:
+            raise ValueError("search_payload must be provided")
+
+        params = {"limit": limit} if limit is not None else {}
+        response = await self.aclient.post(
+            f"/sessions/{session_id}/search",
+            json=search_payload.dict(),
+            params=params,
+        )
+        if response.status_code == 404:
+            raise NotFoundError("No query results found")
+        if response.status_code != 200:
+            raise APIError(response)
+        return [
+            MemorySearchResult(**search_result) for search_result in response.json()
+        ]
 
     async def aclose(self) -> None:
         """
@@ -382,7 +421,7 @@ class ZepClient:
         [Optional] This method may be called when the ZepClient is no longer needed to
         release resources.
         """
-        await self.client.aclose()
+        await self.aclient.aclose()
 
     def close(self) -> None:
         """
@@ -391,4 +430,4 @@ class ZepClient:
         [Optional] This method may be called when the ZepClient is no longer needed to
         release resources.
         """
-        asyncio.run(self.client.aclose())
+        self.client.close()
